@@ -29,10 +29,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 
 #include <gst/interfaces/xoverlay.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libebook/e-book.h>
 #include <glade/glade.h>
 
@@ -117,23 +117,48 @@ typedef struct
 
 } CheeseWindow;
 
+static void 
+open_url(const char *url, GdkScreen *screen)
+{
+  GError *error = NULL;
+  gboolean ret;
+  char *cmdline;
+  GtkWidget *error_dialog;
+
+  cmdline = g_strconcat ("xdg-open ", url, NULL);
+  ret = gdk_spawn_command_line_on_screen (screen, cmdline, &error);
+  g_free (cmdline);
+
+  if (ret == FALSE) {
+    error_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, 
+		 	  		   GTK_MESSAGE_INFO, GTK_BUTTONS_OK, 
+				           _("Failed to show url %s"), error->message); 
+		                           gtk_dialog_run (GTK_DIALOG (error_dialog));
+    g_error_free(error);
+  }
+}
 
 /* Make url in about dialog clickable */
 static void
-cheese_about_dialog_handle_url (GtkAboutDialog *about, const char *link, gpointer data)
+cheese_about_dialog_handle_url (GtkAboutDialog *dialog, const char *url, gpointer data)
 {
-  gnome_vfs_url_show (link);
+  GdkScreen *gscreen;
+  
+  gscreen = gtk_window_get_screen (GTK_WINDOW (dialog));
+  open_url (url, gscreen);
 }
 
 /* Make email in about dialog clickable */
 static void
-cheese_about_dialog_handle_email (GtkAboutDialog *about, const char *link, gpointer data)
+cheese_about_dialog_handle_email (GtkAboutDialog *dialog, const char *email, gpointer data)
 {
-  char *address;
+  char *url;
+  GdkScreen *gscreen;
 
-  address = g_strdup_printf ("mailto:%s", link);
-  gnome_vfs_url_show (address);
-  g_free (address);
+  gscreen = gtk_window_get_screen (GTK_WINDOW (dialog));
+  url = g_strconcat ("mailto:", email, NULL);
+  open_url (url, gscreen);
+  g_free (url);
 }
 
 static char *
@@ -177,6 +202,7 @@ cheese_window_video_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
   gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
 }
 
+
 static void
 cheese_window_cmd_close (GtkWidget *widget, CheeseWindow *cheese_window)
 {
@@ -184,7 +210,17 @@ cheese_window_cmd_close (GtkWidget *widget, CheeseWindow *cheese_window)
   g_object_unref (cheese_window->webcam);
   g_object_unref (cheese_window->actions_main);
   g_object_unref (cheese_window->actions_photo);
+  g_object_unref (cheese_window->actions_mail);
+  g_object_unref (cheese_window->actions_toggle);
+  g_object_unref (cheese_window->actions_effects);
+  g_object_unref (cheese_window->actions_file);
+  g_object_unref (cheese_window->actions_video);
+  g_object_unref (cheese_window->actions_account_photo);
+  g_object_unref (cheese_window->actions_fspot);
+  g_object_unref (cheese_window->actions_flickr);
   g_object_unref (cheese_window->ui_manager);
+  g_object_unref (cheese_window->gconf);
+
   g_free (cheese_window);
   gtk_main_quit ();
 }
@@ -194,13 +230,16 @@ cheese_window_cmd_open (GtkWidget *widget, CheeseWindow *cheese_window)
 {
   char *uri;
   char *filename;
+  GdkScreen *gscreen;
+
+  gscreen = gtk_window_get_screen (GTK_WINDOW (cheese_window->window));
 
   filename = cheese_thumb_view_get_selected_image (CHEESE_THUMB_VIEW (cheese_window->thumb_view));
   g_return_if_fail (filename);
 
-  uri = gnome_vfs_get_uri_from_local_path (filename);
+  uri = g_filename_to_uri (filename, NULL, NULL);
   g_free (filename);
-  gnome_vfs_url_show (uri);
+  open_url (uri, gscreen);
   g_free (uri);
 }
 
@@ -215,7 +254,7 @@ cheese_window_cmd_save_as (GtkWidget *widget, CheeseWindow *cheese_window)
   filename = cheese_thumb_view_get_selected_image (CHEESE_THUMB_VIEW (cheese_window->thumb_view));
   g_return_if_fail (filename);
 
-  dialog = gtk_file_chooser_dialog_new ("Save File",
+  dialog = gtk_file_chooser_dialog_new (_("Save File"),
                                         GTK_WINDOW (cheese_window->window),
                                         GTK_FILE_CHOOSER_ACTION_SAVE,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -234,30 +273,25 @@ cheese_window_cmd_save_as (GtkWidget *widget, CheeseWindow *cheese_window)
   if (response == GTK_RESPONSE_ACCEPT)
   {
     char *target_filename;
+    GError *error = NULL;
+    gboolean ok;
 
     target_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    GFile *target = g_file_new_for_path (target_filename);
 
-    char *source_uri = g_filename_to_uri (filename, NULL, NULL);
-    GnomeVFSURI *source = gnome_vfs_uri_new (source_uri);
-    g_free (source_uri);
+    GFile *source = g_file_new_for_path (filename);
 
-    char *target_uri = g_filename_to_uri (target_filename, NULL, NULL);
-    GnomeVFSURI *target = gnome_vfs_uri_new (target_uri);
-    g_free (target_uri);
+    ok = g_file_copy (source, target, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &error);
 
-    response = gnome_vfs_xfer_uri (source, target,
-                                   GNOME_VFS_XFER_DEFAULT | GNOME_VFS_XFER_FOLLOW_LINKS,
-                                   GNOME_VFS_XFER_ERROR_MODE_ABORT,
-                                   GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-                                   NULL, NULL);
-    gnome_vfs_uri_unref (source);
-    gnome_vfs_uri_unref (target);
+    g_object_unref (source);
+    g_object_unref (target);
 
-    if (response != GNOME_VFS_OK)
+    if (!ok)
     {
       char *header;
       GtkWidget *dlg;
 
+      g_error_free (error);
       header = g_strdup_printf (_("Could not save %s"), target_filename);
 
       dlg = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
@@ -269,65 +303,39 @@ cheese_window_cmd_save_as (GtkWidget *widget, CheeseWindow *cheese_window)
       gtk_widget_destroy (dlg);
       g_free (header);
     }
+    g_free(target_filename);
   }
+  g_free(filename);
   gtk_widget_destroy (dialog);
 }
 
-
 static void
-cheese_window_cmd_move_file_to_trash (CheeseWindow *cheese_window, char *filename)
+cheese_window_cmd_move_file_to_trash (CheeseWindow *cheese_window, GList *files)
 {
-  GnomeVFSURI *uri;
-  GnomeVFSURI *trash_dir;
-  GnomeVFSURI *trash_uri;
-  int result;
-  char *name;
+  GError *error;
+  GList *l;
+  gchar *primary, *secondary;
+  GtkWidget *error_dialog;
 
-  uri = gnome_vfs_uri_new (g_filename_to_uri (filename, NULL, NULL));
-  result = gnome_vfs_find_directory (uri, GNOME_VFS_DIRECTORY_KIND_TRASH,
-                                     &trash_dir, FALSE, FALSE, 0777);
-  if (result != GNOME_VFS_OK)
-  {
-    gnome_vfs_uri_unref (uri);
-    char *header;
-    GtkWidget *dlg;
+  for (l = files; l != NULL; l = l->next) {
+    if (!g_file_trash (l->data, NULL, &error)) {
+      primary = g_strdup (_("Cannot move file to trash"));
+      secondary = g_strdup_printf (_("The file \"%s\" cannot be moved to the trash. Details: %s"),
+				   g_file_get_basename (l->data), error->message);
 
-    header = g_strdup_printf (_("Could not find the Trash"));
-    dlg = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
-                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header);
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), gnome_vfs_result_to_string (result));
-    gtk_dialog_run (GTK_DIALOG (dlg));
-    gtk_widget_destroy (dlg);
-    g_free (header);
-    return;
-  }
+      error_dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
+ 					     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				             GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, primary);
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
+					        secondary);
+      gtk_dialog_run (GTK_DIALOG (error_dialog));
+      gtk_widget_destroy (error_dialog);
 
-  name = gnome_vfs_uri_extract_short_name (uri);
-  trash_uri = gnome_vfs_uri_append_file_name (trash_dir, name);
-  g_free (name);
-
-  result = gnome_vfs_move_uri (uri, trash_uri, TRUE);
-
-  gnome_vfs_uri_unref (uri);
-  gnome_vfs_uri_unref (trash_uri);
-  gnome_vfs_uri_unref (trash_dir);
-
-  if (result != GNOME_VFS_OK)
-  {
-    char *header;
-    GtkWidget *dlg;
-    char *basename = g_path_get_basename (filename);
-
-    header = g_strdup_printf (_("Error on deleting %s"), basename);
-    g_free (basename);
-    dlg = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
-                                  GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header);
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), gnome_vfs_result_to_string (result));
-    gtk_dialog_run (GTK_DIALOG (dlg));
-    gtk_widget_destroy (dlg);
-    g_free (header);
+      g_free (primary);
+      g_free (secondary);
+      g_error_free(error);
+      /*TODO if we can't move files to trash, maybe we should try to delete them....*/
+    }
   }
 }
 
@@ -338,19 +346,21 @@ cheese_window_move_all_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_w
   char *prompt;
   int response;
   char *filename;
+  GFile *file;
+  GList *files_list = NULL;
   GDir *dir;
   char *path;
   const char *name;
 
-  prompt = g_strdup_printf (_("Are you sure you want to move\nall media to the trash?"));
+  prompt = g_strdup_printf (_("Really move all photos and videos to the trash?"));
   dlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (cheese_window->window),
                                             GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                            GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+                                            GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE,
                                             "<span weight=\"bold\" size=\"larger\">%s</span>",
                                             prompt);
   g_free (prompt);
   gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button (GTK_DIALOG (dlg), _("Move to Trash"), GTK_RESPONSE_OK);
+  gtk_dialog_add_button (GTK_DIALOG (dlg), _("_Move to Trash"), GTK_RESPONSE_OK);
   gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
   gtk_window_set_title (GTK_WINDOW (dlg), "");
   gtk_widget_show_all (dlg);
@@ -368,46 +378,34 @@ cheese_window_move_all_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_w
     if (g_str_has_suffix (name, PHOTO_NAME_SUFFIX) || 
         g_str_has_suffix (name, VIDEO_NAME_SUFFIX))
     {
-      filename = g_strjoin ("/", path, name, NULL);
-      cheese_window_cmd_move_file_to_trash (cheese_window, filename);
+      filename = g_strjoin (G_DIR_SEPARATOR_S, path, name, NULL);
+      file = g_file_new_for_path (filename);
+
+      files_list = g_list_append (files_list, file);
       g_free (filename);
     }
   }
+  cheese_window_cmd_move_file_to_trash (cheese_window, files_list);
+  g_list_free (files_list);
+  g_dir_close(dir);
 }
 
 static void
 cheese_window_move_media_to_trash (GtkWidget *widget, CheeseWindow *cheese_window)
 {
-  GtkWidget *dlg;
-  char *prompt;
-  int response;
-  char *filename, *basename;
+  char *filename;
+  GFile *file;
+  GList *files_list = NULL;
 
   filename = cheese_thumb_view_get_selected_image (CHEESE_THUMB_VIEW (cheese_window->thumb_view));
   g_return_if_fail (filename);
 
-  basename = g_path_get_basename (filename);
-  prompt = g_strdup_printf (_("Are you sure you want to move\n\"%s\" to the trash?"), basename);
-  g_free (basename);
-  dlg = gtk_message_dialog_new_with_markup (GTK_WINDOW (cheese_window->window),
-                                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                            GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-                                            "<span weight=\"bold\" size=\"larger\">%s</span>",
-                                            prompt);
-  g_free (prompt);
-  gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-  gtk_dialog_add_button (GTK_DIALOG (dlg), _("Move to Trash"), GTK_RESPONSE_OK);
-  gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_OK);
-  gtk_window_set_title (GTK_WINDOW (dlg), "");
-  gtk_widget_show_all (dlg);
+  file = g_file_new_for_path (filename);
+  g_free(filename);
 
-  response = gtk_dialog_run (GTK_DIALOG (dlg));
-  gtk_widget_destroy (dlg);
-
-  if (response !=  GTK_RESPONSE_OK)
-    return;
-
-  cheese_window_cmd_move_file_to_trash (cheese_window, filename);
+  files_list = g_list_append (files_list, file);
+  cheese_window_cmd_move_file_to_trash (cheese_window, files_list);
+  g_list_free (files_list);
 }
 
 static void
@@ -458,13 +456,16 @@ cheese_window_cmd_set_about_me_photo (GtkWidget *widget, CheeseWindow *cheese_wi
         gtk_dialog_run (GTK_DIALOG (dlg));
         gtk_widget_destroy (dlg);
         g_free (header);
+        g_error_free (error);
       }
       g_free (*data);
     }
+    g_free(filename);
     g_object_unref (pixbuf);
   }
 }
 
+
 static void
 cheese_window_cmd_command_line (GtkAction *action, CheeseWindow *cheese_window)
 {
@@ -792,7 +793,7 @@ cheese_window_countdown_picture_cb (gpointer data)
   if (!audio_play) 
   {
     g_warning (error ? error->message : "Unknown error");
-    g_clear_error (&error);
+    g_error_free (error);
   }
 
   g_free (file);
@@ -1109,11 +1110,33 @@ cheese_window_create_window (CheeseWindow *cheese_window)
                     G_CALLBACK (cheese_window_button_press_event_cb), cheese_window);
 }
 
+void 
+setup_camera (CheeseWindow *cheese_window)
+{
+  char *webcam_device = NULL;
+
+  g_object_get (cheese_window->gconf, "gconf_prop_webcam", &webcam_device, NULL);
+
+  cheese_window->webcam = cheese_webcam_new (cheese_window->screen, webcam_device);
+  g_free (webcam_device);
+
+  g_signal_connect (cheese_window->webcam, "photo-saved",
+                    G_CALLBACK (cheese_window_photo_saved_cb), cheese_window);
+  g_signal_connect (cheese_window->webcam, "video-saved",
+                    G_CALLBACK (cheese_window_video_saved_cb), cheese_window);
+
+  cheese_webcam_set_effect (cheese_window->webcam, 
+                            cheese_effect_chooser_get_selection (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)));
+
+  cheese_webcam_play (cheese_window->webcam);
+  gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
+  ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
+}
+
 void
 cheese_window_init ()
 {
   CheeseWindow *cheese_window;
-  char *webcam_device = NULL;
 
   cheese_window = g_new (CheeseWindow, 1);
 
@@ -1131,25 +1154,15 @@ cheese_window_init ()
   cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
   cheese_window->recording = FALSE;
   
-  g_object_get (cheese_window->gconf, "gconf_prop_webcam", &webcam_device, NULL);
-
-  cheese_window->webcam = cheese_webcam_new (cheese_window->screen, webcam_device);
-  g_free (webcam_device);
-
-  g_signal_connect (cheese_window->webcam, "photo-saved",
-                    G_CALLBACK (cheese_window_photo_saved_cb), cheese_window);
-  g_signal_connect (cheese_window->webcam, "video-saved",
-                    G_CALLBACK (cheese_window_video_saved_cb), cheese_window);
+  /* Run cam setup in its own thread */
+  GError *error = NULL;
+  if (!g_thread_create ((GThreadFunc) setup_camera, cheese_window, FALSE, &error)) {
+    g_error ("Failed to create setup thread: %s\n", error->message);
+    g_error_free (error);
+    return;
+  }
 
   /* Make URLs and email clickable in about dialog */
   gtk_about_dialog_set_url_hook (cheese_about_dialog_handle_url, NULL, NULL);
   gtk_about_dialog_set_email_hook (cheese_about_dialog_handle_email, NULL, NULL);
-
-  cheese_webcam_set_effect (cheese_window->webcam, 
-                            cheese_effect_chooser_get_selection (CHEESE_EFFECT_CHOOSER (cheese_window->effect_chooser)));
-
-  cheese_webcam_play (cheese_window->webcam);
-  gtk_notebook_set_current_page (GTK_NOTEBOOK(cheese_window->notebook), 0);
-  ephy_spinner_stop (EPHY_SPINNER (cheese_window->throbber));
 }
-
